//
//  TwitterEngine.h
//  TwitterEngine
//
//  Created by Xiao Xiao on 11-12-17.
//  Copyright (c) 2011年 Xiao Xiao. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <Twitter/Twitter.h>
#import <Accounts/Accounts.h>
#import "TwitterEngineGlobalHeader.h"

typedef NSUInteger(^ACAccountSelectionHandler)(NSArray *accountsArray, NSError *error);

@interface TwitterEngine : NSObject
{
    ACAccount *activeAccount;
    BOOL activated;
}

/** Fetch the local Twitter accounts list. if multiple accounts exist, the selection handler will be called to do a selection; whereelse the only account will be selected. If no account exists, returns NO. **/
- (BOOL)activateWithAccount:(ACAccount *)account;

/** Timelines */

- (void)getPublicTimelineWithTrimUser:(BOOL)shouldTrimUser includeEntities:(BOOL)shouldIncludeEntities;

- (void)getHomeTimelineWithCount:(NSUInteger)count sinceID:(TweetID)sinceID maxID:(TweetID)maxID trimUser:(BOOL)shouldTrimUser includeRetweets:(BOOL)shouldIncludeRetweets includeEntities:(BOOL)shouldIncludeEntities excludeReplies:(BOOL)shouldExcludeReplies contributorDetails:(BOOL)shouldIncludeContributorDetails;

- (void)getMentionsWithCount:(NSUInteger)count sinceID:(TweetID)sinceID maxID:(TweetID)maxID trimUser:(BOOL)shouldTrimUser includeRetweets:(BOOL)shouldIncludeRetweets includeEntities:(BOOL)shouldIncludeEntities excludeReplies:(BOOL)shouldExcludeReplies contributorDetails:(BOOL)shouldIncludeContributorDetails;

- (void)getUserTimelineForUserID:(UserID)userID orScreenName:(NSString *)screenName withCount:(NSUInteger)count sinceID:(TweetID)sinceID maxID:(TweetID)maxID trimUser:(BOOL)shouldTrimUser includeRetweets:(BOOL)shouldIncludeRetweets includeEntities:(BOOL)shouldIncludeEntities excludeReplies:(BOOL)shouldExcludeReplies contributorDetails:(BOOL)shouldIncludeContributorDetails;

/** Tweets */

- (void)getRetweetByForTweetID:(TweetID)ID WithCount:(NSUInteger)count;

- (void)getShowForTweetID:(TweetID)ID withTrimUser:(BOOL)shouldTrimUser includeEntities:(BOOL)shouldIncludeEntities;

//- (void)postDestroyForTweetID:(TweetID)ID withTrimUser:(BOOL)shouldTrimUser includeEntities:(BOOL)shouldIncludeEntities;
//
//- (void)postRetweetForTweetID:(TweetID)ID withTrimUser:(BOOL)shouldTrimUser includeEntities:(BOOL)shouldIncludeEntities;
//
- (void)postUpdateForStatus:(NSString *)status inReplyToStatusID:(TweetID)ID withLatitude:(TWLocationDegrees)latitude longitude:(TWLocationDegrees)longitude withTrimUser:(BOOL)shouldTrimUser includeEntities:(BOOL)shouldIncludeEntities wrapLinks:(BOOL)shouldWrapLinks;

//- (void)postUpdateWithMediaForStatus:(NSString *)status  inReplyToStatusID:(TweetID)ID withMedia:(NSArray *)media possiblySensitive:(BOOL)isPossiblySensitive latitude:(TWLocationDegrees)latitude longitude:(TWLocationDegrees)longitude;

///** Search */

- (void)getSearchForQuery:(NSString *)query withLanguage:(NSString *)langcode resultType:(NSString *)resultType includeEntities:(BOOL)shouldIncludeEntities;

/** Friends & Followers */
- (void)getFollowersForUserID:(UserID)userID orScreenName:(NSString *)screenName;

- (void)getFriendsForUserID:(UserID)userID orScreenName:(NSString *)screenName;

//- (void)getFriendshipsIncoming;
//
//- (void)getFriendshipsOutgoing;
//
//- (void)getFriendshipsShowForSourceUserID:(UserID)sourceID orScreenName:(NSString *)sourceScreenName withTargetUserID:(UserID)targetID orScreenName:(NSString *)targetScreenName;
//
//- (void)postFriendshipsCreateWithUserID:(UserID)userID orScreenName:(NSString *)screenName;
//
//- (void)postFriendshipsDestroyWithUserID:(UserID)userID orScreenName:(NSString *)screenName includeEntities:(BOOL)shouldIncludeEntities;

/** Users */

- (void)getUsersLookupWithUserID:(UserID)userID orScreenName:(NSString *)screenName includeEntities:(BOOL)shouldIncludeEntities;

//- (void)getProfileImageForScreenName:(NSString *)screenName size:(NSString *)size;
//
//- (void)getUsersSearchForQuery:(NSString *)query includeEntities:(BOOL)shouldIncludeEntities;

- (void)getUsersShowWithUserID:(UserID)userID orScreenName:(NSString *)screenName includeEntities:(BOOL)shouldIncludeEntities;

/** Favorites */

- (void)getFavoritesForUserID:(UserID)userID sinceID:(TweetID)sinceID includeEntities:(BOOL)shouldIncludeEntities;

//- (void)postFavoritesCreateForTweetID:(TweetID)tweetID includeEntities:(BOOL)shouldIncludeEntities;
//
//- (void)postFavoritesDestroyForTweetID:(TweetID)tweetID;

/** Accounts */

//- (void)getAccountRateLimitStatus;
//
//- (void)postUpdateProfileWithName:(NSString *)name URL:(NSString *)url location:(NSString *)location description:(NSString *)description includeEntities:(BOOL)shouldIncludeEntities skipStatus:(BOOL)shouldSkipStatus;
//
//- (void)postUpdateProfileBackgroundImageWithImage:(UIImage *)image tile:(BOOL)shouldTile includeEntities:(BOOL)shouldIncludeEntities skipStatus:(BOOL)shouldSkipStatus;
//
//- (void)postUpdateProfileImageWithImage:(UIImage *)image includeEntities:(BOOL)shouldIncludeEntities skipStatus:(BOOL)shouldSkipStatus;
//
//- (void)getAccountTotals;


/** Append request with selected account and perform on another thread. Handler will be called on main thread. */
- (void)performRequest:(TWRequest *)request withResultNotificationName:(NSString *)name;

/** Dispatch the API returned data in a form of nsnotification userinfo. */
- (void)dispatchResultNotification:(NSDictionary *)dispatchInfo;

@end
