//
//  TwitterEngine.m
//  TwitterEngine
//
//  Created by Xiao Xiao on 11-12-17.
//  Copyright (c) 2011年 Xiao Xiao. All rights reserved.
//

#import "TwitterEngine.h"
#import "TwitterRequestURLs.h"
#import "NSNumber+TwitterAdditions.h"
#import "NSString+TwitterAdditions.h"
#import "NSDictionary+TwitterAdditions.h"
#import "NSURL+SSToolkitAdditions.h"

@implementation TwitterEngine

- (id)init
{
    self = [super init];
    if (self) {
        // Initialization code here.
        activated = NO;
    }
    return self;
}

- (BOOL)activateWithAccount:(ACAccount *)account
{
//    // Create an account store object.
//	accountStore = [[ACAccountStore alloc] init];
//	
//	// Create an account type that ensures Twitter accounts are retrieved.
//    accountType = [accountStore accountTypeWithAccountTypeIdentifier:ACAccountTypeIdentifierTwitter];
//    
//    [accountStore requestAccessToAccountsWithType:accountType withCompletionHandler:^(BOOL granted, NSError *error) {
//        if(granted) {
//            [self performSelectorOnMainThread:@selector(setAccount) withObject:nil waitUntilDone:YES];
//        }
//    }];
    activeAccount = account;
    activated = YES;
    return activated;
}
//
//- (void)setAccount{
//    // Get the list of Twitter accounts.
//    NSArray *accountsArray = [accountStore accountsWithAccountType:accountType];
//    NSLog(@"%d", [accountsArray count]);
//    
//    self.activeAccount = [accountsArray objectAtIndex:0];
//    NSLog(@"active Account temp is:%@",activeAccount);
//}

#pragma mark -
#pragma mark Timeline
- (void)getPublicTimelineWithTrimUser:(BOOL)shouldTrimUser includeEntities:(BOOL)shouldIncludeEntities
{
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeysExcludeNull:[NSString stringWithBool:shouldTrimUser], @"trim_user", [NSString stringWithBool:shouldIncludeEntities], @"include_entities", nil];
    TWRequest *getRequest = [[TWRequest alloc]initWithURL:[NSURL URLWithString:TwitterPublicTimelineURL] parameters:parameters requestMethod:TWRequestMethodGET];
    [self performRequest:getRequest withResultNotificationName:@"PublicTimeline"];
}

- (void)getHomeTimelineWithCount:(NSUInteger)count sinceID:(TweetID)sinceID maxID:(TweetID)maxID trimUser:(BOOL)shouldTrimUser includeRetweets:(BOOL)shouldIncludeRetweets includeEntities:(BOOL)shouldIncludeEntities excludeReplies:(BOOL)shouldExcludeReplies contributorDetails:(BOOL)shouldIncludeContributorDetails
{
    
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeysExcludeNull:[NSString stringWithUInteger:count], @"count", [NSString stringWithID:sinceID], @"since_id", [NSString stringWithID:maxID], @"max_id", [NSString stringWithBool:shouldTrimUser], @"trim_user", [NSString stringWithBool:shouldIncludeRetweets], @"include_rts", [NSString stringWithBool:shouldIncludeEntities], @"include_entities", [NSString stringWithBool:shouldExcludeReplies], @"exclude_replies", [NSString stringWithBool:shouldIncludeContributorDetails], @"contributor_details", nil];
    TWRequest *getRequest = [[TWRequest alloc]initWithURL:[NSURL URLWithString:TwitterHomeTimelineURL] parameters:(NSDictionary *)parameters requestMethod:TWRequestMethodGET];
    [self performRequest:getRequest withResultNotificationName:@"HomeTimeline"];
}

- (void)getMentionsWithCount:(NSUInteger)count sinceID:(TweetID)sinceID maxID:(TweetID)maxID trimUser:(BOOL)shouldTrimUser includeRetweets:(BOOL)shouldIncludeRetweets includeEntities:(BOOL)shouldIncludeEntities excludeReplies:(BOOL)shouldExcludeReplies contributorDetails:(BOOL)shouldIncludeContributorDetails
{
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeysExcludeNull:[NSString stringWithUInteger:count], @"count", [NSString stringWithID:sinceID], @"since_id", [NSString stringWithID:maxID], @"max_id", [NSString stringWithBool:shouldTrimUser], @"trim_user", [NSString stringWithBool:shouldIncludeRetweets], @"include_rts", [NSString stringWithBool:shouldIncludeEntities], @"include_entities", [NSString stringWithBool:shouldExcludeReplies], @"exclude_replies", [NSString stringWithBool:shouldIncludeContributorDetails], @"contributor_details", nil];
    TWRequest *getRequest = [[TWRequest alloc]initWithURL:[NSURL URLWithString:TwitterMentionsURL] parameters:(NSDictionary *)parameters requestMethod:TWRequestMethodGET];
    [self performRequest:getRequest withResultNotificationName:@"Mentions"];
}

- (void)getUserTimelineForUserID:(UserID)userID orScreenName:(NSString *)screenName withCount:(NSUInteger)count sinceID:(TweetID)sinceID maxID:(TweetID)maxID trimUser:(BOOL)shouldTrimUser includeRetweets:(BOOL)shouldIncludeRetweets includeEntities:(BOOL)shouldIncludeEntities excludeReplies:(BOOL)shouldExcludeReplies contributorDetails:(BOOL)shouldIncludeContributorDetails
{
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeysExcludeNull:[NSString stringWithID:userID], @"user_id", screenName, @"screen_name" , [NSString stringWithUInteger:count], @"count", [NSString stringWithID:sinceID], @"since_id", [NSString stringWithID:maxID], @"max_id", [NSString stringWithBool:shouldTrimUser], @"trim_user", [NSString stringWithBool:shouldIncludeRetweets], @"include_rts", [NSString stringWithBool:shouldIncludeEntities], @"include_entities", [NSString stringWithBool:shouldExcludeReplies], @"exclude_replies", [NSString stringWithBool:shouldIncludeContributorDetails], @"contributor_details", nil];
    TWRequest *getRequest = [[TWRequest alloc]initWithURL:[NSURL URLWithString:TwitterMentionsURL] parameters:(NSDictionary *)parameters requestMethod:TWRequestMethodGET];
    [self performRequest:getRequest withResultNotificationName:@"UserTimeline"];
}

#pragma mark -
#pragma mark Tweet
- (void)getRetweetByForTweetID:(TweetID)ID WithCount:(NSUInteger)count
{
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeysExcludeNull:[NSString stringWithUInteger:count], @"count", nil];
    TWRequest *getRequest = [[TWRequest alloc]initWithURL:[NSURL URLWithFormat:TwitterRetweetByURL,[NSNumber numberWithID:ID]] parameters:(NSDictionary *)parameters requestMethod:TWRequestMethodGET];
    [self performRequest:getRequest withResultNotificationName:@"RetweetBy"];
}

- (void)getShowForTweetID:(TweetID)ID withTrimUser:(BOOL)shouldTrimUser includeEntities:(BOOL)shouldIncludeEntities
{
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeysExcludeNull:[NSString stringWithBool:shouldTrimUser], @"trim_user", [NSString stringWithBool:shouldIncludeEntities], @"include_entities", nil];
    TWRequest *getRequest = [[TWRequest alloc]initWithURL:[NSURL URLWithFormat:TwitterMentionsURL, [NSNumber numberWithID:ID]] parameters:(NSDictionary *)parameters requestMethod:TWRequestMethodGET];
    [self performRequest:getRequest withResultNotificationName:@"UserTimeline"];
}

- (void)postUpdateForStatus:(NSString *)status inReplyToStatusID:(TweetID)ID withLatitude:(TWLocationDegrees)latitude longitude:(TWLocationDegrees)longitude withTrimUser:(BOOL)shouldTrimUser includeEntities:(BOOL)shouldIncludeEntities wrapLinks:(BOOL)shouldWrapLinks
{
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeysExcludeNull:status, @"status", [NSString stringWithID:ID], @"in_reply_to_status_id", [NSString stringWithDegrees:latitude], @"latitude", [NSString stringWithDegrees:longitude], @"longitude", [NSString stringWithBool:shouldTrimUser], @"trim_user", [NSString stringWithBool:shouldIncludeEntities], @"include_entities", [NSString stringWithBool:shouldWrapLinks], @"wrap_links", nil];
    TWRequest *getRequest = [[TWRequest alloc]initWithURL:[NSURL URLWithString:TwitterUpdateURL] parameters:(NSDictionary *)parameters requestMethod:TWRequestMethodPOST];
    [self performRequest:getRequest withResultNotificationName:@"Update"];
}

#pragma mark -
#pragma mark Search
- (void)getSearchForQuery:(NSString *)query withLanguage:(NSString *)langcode resultType:(NSString *)resultType includeEntities:(BOOL)shouldIncludeEntities
{
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeysExcludeNull:query, @"query", langcode, @"lang", resultType, @"result_type", [NSString stringWithBool:shouldIncludeEntities], @"include_entities", nil];
    TWRequest *getRequest = [[TWRequest alloc]initWithURL:[NSURL URLWithString:TwitterSearchURL] parameters:(NSDictionary *)parameters requestMethod:TWRequestMethodGET];
    [self performRequest:getRequest withResultNotificationName:@"Search"];
}

#pragma mark -
#pragma mark Friends & Followers
- (void)getFollowersForUserID:(UserID)userID orScreenName:(NSString *)screenName
{
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeysExcludeNull:[NSString stringWithID:userID], @"user_id", screenName, @"screen_name", nil];
    TWRequest *getRequest = [[TWRequest alloc]initWithURL:[NSURL URLWithString:TwitterFollowersIDsURL] parameters:(NSDictionary *)parameters requestMethod:TWRequestMethodGET];
    [self performRequest:getRequest withResultNotificationName:@"Followers"];
}

- (void)getFriendsForUserID:(UserID)userID orScreenName:(NSString *)screenName
{
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeysExcludeNull:[NSString stringWithID:userID], @"user_id", screenName, @"screen_name", nil];
    TWRequest *getRequest = [[TWRequest alloc]initWithURL:[NSURL URLWithString:TwitterFriendsIDsURL] parameters:(NSDictionary *)parameters requestMethod:TWRequestMethodGET];
    [self performRequest:getRequest withResultNotificationName:@"Friends"];
}

#pragma mark -
#pragma mark Users
- (void)getUsersLookupWithUserID:(UserID)userID orScreenName:(NSString *)screenName includeEntities:(BOOL)shouldIncludeEntities
{
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeysExcludeNull:[NSString stringWithID:userID], @"user_id", screenName, @"screen_name", [NSString stringWithBool:shouldIncludeEntities], @"include_entities", nil];
    TWRequest *getRequest = [[TWRequest alloc]initWithURL:[NSURL URLWithString:TwitterUsersLookupURL] parameters:(NSDictionary *)parameters requestMethod:TWRequestMethodGET];
    [self performRequest:getRequest withResultNotificationName:@"UsersLookup"];
}

- (void)getUsersShowWithUserID:(UserID)userID orScreenName:(NSString *)screenName includeEntities:(BOOL)shouldIncludeEntities
{
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeysExcludeNull:[NSString stringWithID:userID], @"user_id", screenName, @"screen_name", [NSString stringWithBool:shouldIncludeEntities], @"include_entities", nil];
    TWRequest *getRequest = [[TWRequest alloc]initWithURL:[NSURL URLWithString:TwitterUsersShowURL] parameters:(NSDictionary *)parameters requestMethod:TWRequestMethodGET];
    [self performRequest:getRequest withResultNotificationName:@"UsersShow"];
}

#pragma mark -
#pragma mark Favorites
- (void)getFavoritesForUserID:(UserID)userID sinceID:(TweetID)sinceID includeEntities:(BOOL)shouldIncludeEntities
{
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeysExcludeNull:[NSString stringWithID:userID], @"user_id",[NSString stringWithID:sinceID], @"since_id", [NSString stringWithBool:shouldIncludeEntities], @"include_entities", nil];
    TWRequest *getRequest = [[TWRequest alloc]initWithURL:[NSURL URLWithString:TwitterFavoritesURL] parameters:(NSDictionary *)parameters requestMethod:TWRequestMethodGET];
    [self performRequest:getRequest withResultNotificationName:@"Favorites"];
}

#pragma mark -
#pragma mark Request dispatch & notification
- (void)performRequest:(TWRequest *)request withResultNotificationName:(NSString *)name
{
    __block NSString *resultNotificationName = name;
    void (^requestHandler)(NSData *, NSHTTPURLResponse *, NSError *) = ^(NSData *responseData, NSHTTPURLResponse *urlResponse, NSError *error){
        NSError *jsonParsingError = nil;
        NSArray *resultArray = [NSJSONSerialization JSONObjectWithData:responseData options:0 error:&jsonParsingError];
        NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:resultNotificationName, @"NotificationName", resultArray, @"result", nil];
        [self performSelectorOnMainThread:@selector(dispatchResultNotification:) withObject:userInfo waitUntilDone:NO];
    };
    NSLog(@"active Account is:%@",activeAccount);
    [request setAccount:activeAccount];
    [request performRequestWithHandler:requestHandler];
}

- (void)dispatchResultNotification:(NSDictionary *)dispatchInfo
{
//    NSNotification *resultNotification = [NSNotification notificationWithName:[dispatchInfo objectForKey:@"NotificationName"]  object:self userInfo:dispatchInfo];
    [[NSNotificationCenter defaultCenter]postNotificationName:[dispatchInfo objectForKey:@"NotificationName"] object:self userInfo:dispatchInfo];
}

@end
